"""Fancy lyrics managment."""

import logging

from .extractor import LyricsExtractor
from .models import LyricsOrigin, exceptions
from .utils import UrlData, search

log = logging.getLogger(__name__)


class LyricsManager:
    """Manage stuff."""

    extractors = []

    google_api_key = None

    @classmethod
    def setup(cls):
        """Load extractors."""
        log.debug("setting up")
        from . import extractors  # noqa F401
        cls.extractors = LyricsExtractor.extractors
        log.info("loaded {} extractors".format(len(cls.extractors)))

    @classmethod
    def extract_lyrics(cls, url):
        """Extract lyrics from url."""
        log.info("extracting lyrics from url \"{}\"".format(url))
        url_data = UrlData(url)
        for extractor in cls.extractors:
            if extractor.can_handle(url_data):
                log.debug("using {} for {}".format(extractor, url_data))
                try:
                    lyrics = extractor.extract_lyrics(url_data)
                except exceptions.NoLyrics:
                    log.warn("{} didn't find any lyrics at {}".format(extractor, url))
                    continue
                except exceptions.NotAllowedError:
                    log.warn("{} couldn't access lyrics at {}".format(extractor, url))
                    continue
                except Exception:
                    log.exception("Something went wrong when {} handled {}".format(extractor, url_data))
                    continue

                if lyrics:
                    lyrics.origin = LyricsOrigin(url, extractor.name, extractor.url)
                    log.debug("extracted lyrics {}".format(lyrics))
                    return lyrics
        raise exceptions.NoExtractorError(url)

    @classmethod
    def search_lyrics(cls, query, *, google_api_key):
        """Search the net for lyrics."""
        results = search(query, google_api_key)
        log.debug("got {} results".format(len(results)))
        for result in results:
            url = result["link"]
            try:
                lyrics = cls.extract_lyrics(url)
            except exceptions.NoExtractorError:
                log.warn("No extractor for url {}".format(url))
                continue
            if lyrics:
                lyrics.origin.query = query
                return lyrics
        log.warn("No lyrics found for query \"{}\"".format(query))


LyricsManager.setup()



"""Lyric finder exceptions."""


class LyricsException(Exception):
    """The base class for every lyrics exception."""

    pass


class NoExtractorError(LyricsException):
    """When there's no extractor for a url."""

    def __init__(self, url):
        """Create new."""
        super().__init__("No extractor found for {}".format(url))


class NoLyrics(LyricsException):
    """When this url doesn't point to actual lyrics."""

    pass


class NotAllowedError(LyricsException):
    """When the access is blocked."""

    pass



"""Extractor for Musixmatch.com."""

import logging

from requests.exceptions import HTTPError

from ..extractor import LyricsExtractor
from ..models import exceptions
from ..models.lyrics import Lyrics

log = logging.getLogger(__name__)


class MusixMatch(LyricsExtractor):
    """Class for extracting lyrics."""

    name = "MusixMatch"
    url = "https://www.musixmatch.com/"
    display_url = "musixmatch.com"

    @classmethod
    def extract_lyrics(cls, url_data):
        """Extract lyrics."""
        url_data.headers = {"user-agent": "Mozilla/5.0 (Windows NT 6.1; WOW64; rv:40.0) Gecko/20100101 Firefox/40.1"}
        try:
            url_data.resp.raise_for_status()
        except HTTPError:
            raise exceptions.NotAllowedError
        bs = url_data.bs

        if bs.find_all("div", attrs={"class": "mxm-empty-state", "data-reactid": "87"}):
            raise exceptions.NoLyrics

        lyrics_frame = bs.find_all("div", {"class": "mxm-lyrics"})

        if not lyrics_frame:
            raise exceptions.NoLyrics

        lyrics_window = lyrics_frame[0].find_all("div", {"class": "mxm-lyrics"})

        if not lyrics_window:
            raise exceptions.NoLyrics

        lyrics_window = lyrics_window[0].span

        for garbage in bs.find_all("script"):
            garbage.replace_with(2 * "\n")

        lyrics = lyrics_window.text
        title = bs.find("h1", attrs={"class": "mxm-track-title__track"}).contents[-1].strip()

        return Lyrics(title, lyrics)
